package Lingua::Base::Nums2Words;
# ABSTRACT: Base class for Lingua::ID::Nums2Words and others


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Lingua::Base::Nums2Words - Base class for Lingua::ID::Nums2Words and others

=head1 VERSION

This document describes version 0.04 of Lingua::Base::Nums2Words (from Perl distribution Lingua-ID-Nums2Words), released on 2015-09-03.

=head2 DESCRIPTION

This class can be used as a base for number-to-words converter in any language
that uses decimal system and fixed cluster size.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Lingua-ID-Nums2Words>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Lingua-ID-Nums2Words>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Lingua-ID-Nums2Words>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
